@@ -28,6 +28,8 @@ using System;
 using System.Collections;
 using System.Text;
 using System.Threading;
+using System.Xml;
+using System.Xml.Serialization;
 
 namespace Beagle.Util {
 
@@ -37,7 +39,7 @@ namespace Beagle.Util {
 		public delegate void EmptyQueueDelegate ();
 		public event EmptyQueueDelegate EmptyQueueEvent;
 
-		static public bool Debug = false;
+		public static bool Debug = false;
 
 		public enum Priority {
 
@@ -265,12 +267,13 @@ namespace Beagle.Util {
 					try {
 						DoTaskReal ();
 					} catch (Exception ex) {
-						Logger.Log.Warn ("Caught exception in DoTaskReal");
-						Logger.Log.Warn ("        Tag: {0}", Tag);
-						Logger.Log.Warn ("    Creator: {0}", Creator);
-						Logger.Log.Warn ("Description: {0}", Description);
-						Logger.Log.Warn ("   Priority: {0} ({1})", Priority, SubPriority);
-						Logger.Log.Warn (ex);
+						Logger.Log.Warn (ex,
+								 "Caught exception in DoTaskReal\n" +
+								 "        Tag: {0}\n" +
+								 "    Creator: {0}\n" +
+								 "Description: {0}\n" +
+								 "   Priority: {0} ({1})", 
+								 Tag, Creator, Description, Priority, SubPriority);
 					}
 					sw.Stop ();
 					if (Debug)
@@ -278,6 +281,8 @@ namespace Beagle.Util {
 
 					if (Reschedule) {
 						++count;
+						if (Debug)
+							Log.Debug ("Rescheduling task {0}", Tag);
 						scheduler.Add (this); // re-add ourselves
 					} else {
 						DecrementAllTaskGroups ();
@@ -300,8 +305,7 @@ namespace Beagle.Util {
 				try {
 					DoCleanup ();
 				} catch (Exception ex) {
-					Logger.Log.Warn ("Caught exception cleaning up task '{0}'", Tag);
-					Logger.Log.Warn (ex);
+					Logger.Log.Warn (ex, "Caught exception cleaning up task '{0}'", Tag);
 				}
 			}
 
@@ -463,8 +467,7 @@ namespace Beagle.Util {
 						try {
 							pre_hook ();
 						} catch (Exception ex) {
-							Logger.Log.Warn ("Caught exception in pre_hook of task group '{0}'", Name);
-							Logger.Log.Warn (ex);
+							Logger.Log.Warn (ex, "Caught exception in pre_hook of task group '{0}'", Name);
 						}
 					}
 					touched = true;
@@ -485,8 +488,7 @@ namespace Beagle.Util {
 						try {
 							post_hook ();
 						} catch (Exception ex) {
-							Logger.Log.Warn ("Caught exception in post_hook of task group '{0}'", Name);
-							Logger.Log.Warn (ex);
+							Logger.Log.Warn (ex, "Caught exception in post_hook of task group '{0}'", Name);
 						}
 					}
 					finished = true;
@@ -513,7 +515,7 @@ namespace Beagle.Util {
 
 		//////////////////////////////////////////////////////////////////////////////
 
-		static private double global_delay = -1.0;
+		private static double global_delay = -1.0;
 
 		static Scheduler ()
 		{
@@ -521,6 +523,8 @@ namespace Beagle.Util {
 			exercise = Environment.GetEnvironmentVariable ("BEAGLE_EXERCISE_THE_DOG");
 
 			if (exercise != null) {
+				Log.Info ("BEAGLE_EXERCISE_THE_DOG is set.");
+
 				if (exercise.Length > 2 && exercise [0] == 't')
 					global_delay = Double.Parse (exercise.Substring (1));
 				else
@@ -530,9 +534,9 @@ namespace Beagle.Util {
 
 		//////////////////////////////////////////////////////////////////////////////
 
-		static private Scheduler global = new Scheduler ();
+		private static Scheduler global = new Scheduler ();
 
-		static public Scheduler Global {
+		public static Scheduler Global {
 			get { return global; }
 		}
 
@@ -577,10 +581,9 @@ namespace Beagle.Util {
 					return;
 
 				if (Debug) {
-					Logger.Log.Debug ("Adding task");
-					Logger.Log.Debug ("Tag: {0}", task.Tag);
+					Logger.Log.Debug ("Adding task {0}", task.Tag);
 					if (task.Description != null)
-						Logger.Log.Debug ("Desc: {0}", task.Description);
+						Logger.Log.Debug ("  Desc: {0}", task.Description);
 				}
 
 				if (task.Priority == Priority.Shutdown)
@@ -619,29 +622,39 @@ namespace Beagle.Util {
 
 		//////////////////////////////////////////////////////////////////////////////
 
-		Thread thread = null;
+		private Thread thread = null;
 		public bool running = false;
+		private static bool shutdown_requested = false;
 
 		public void Start ()
 		{
 			lock (this) {
-				if (thread != null)
+				if (shutdown_requested || thread != null)
 					return;
 				running = true;
 				thread = ExceptionHandlingThread.Start (new ThreadStart (Worker));
 			}
 		}
 
-		public void Stop ()
+		public void Stop (bool to_shutdown)
 		{
 			lock (big_lock) {
+				shutdown_requested = to_shutdown;
+
 				if (running) {
 					running = false;
+					thread = null;
+					status_str = "Stopped";
 					Monitor.Pulse (big_lock);
 				}
 			}
 		}
 
+		public void Stop ()
+		{
+			Stop (false);
+		}
+
 		//
 		// Delay Computations
 		//
@@ -971,8 +984,7 @@ namespace Beagle.Util {
 					try {
 						pre_hook ();
 					} catch (Exception ex) {
-						Logger.Log.Error ("Caught exception in pre_hook '{0}'", pre_hook);
-						Logger.Log.Error (ex);
+						Logger.Log.Error (ex, "Caught exception in pre_hook '{0}'", pre_hook);
 					}
 				}
 				foreach (Task task in to_be_executed) {
@@ -984,8 +996,7 @@ namespace Beagle.Util {
 					try {
 						post_hook ();
 					} catch (Exception ex) {
-						Logger.Log.Error ("Caught exception in post_hook '{0}'", post_hook);
-						Logger.Log.Error (ex);
+						Logger.Log.Error (ex, "Caught exception in post_hook '{0}'", post_hook);
 					}
 				}
 
@@ -1008,10 +1019,12 @@ namespace Beagle.Util {
 		
 		//////////////////////////////////////////////////////////////////////////////
 
-		public string GetHumanReadableStatus ()
+		private static StringBuilder cached_sb = new StringBuilder ();
+		
+		public SchedulerInformation GetCurrentStatus ()
 		{
-			StringBuilder sb = new StringBuilder ();
-
+		    SchedulerInformation current_status = new SchedulerInformation ();
+			
 			lock (big_lock) {
 				
 				ArrayList blocked_tasks = new ArrayList ();
@@ -1037,43 +1050,89 @@ namespace Beagle.Util {
 				pending_tasks.Sort ();
 				pending_tasks.Reverse ();
 
-				sb.Append ("Scheduler:\n");
-				sb.Append ("Count: ").Append (total_executed_task_count);
-				sb.Append ('\n');
-
-				if (status_str != null)
-					sb.Append ("Status: ").Append (status_str).Append ('\n');
-
-				int pos = 1;
-				sb.Append ("\nPending Tasks:\n");
 				foreach (Task task in pending_tasks) {
-					sb.Append (pos).Append (' ');
-					task.AppendToStringBuilder (sb);
-					sb.Append ('\n');
-					++pos;
-				}
-				if (pos == 1)
-					sb.Append ("Scheduler queue is empty.\n");
-
-
-				if (future_tasks.Count > 0) {
-					sb.Append ("\nFuture Tasks:\n");
-					foreach (Task task in future_tasks) {
-						task.AppendToStringBuilder (sb);
-						sb.Append ('\n');
-					}
+					cached_sb.Length = 0;
+					task.AppendToStringBuilder (cached_sb);
+					current_status.PendingTasks.Add (cached_sb.ToString ());
 				}
 
-				if (blocked_tasks.Count > 0) {
-					sb.Append ("\nBlocked Tasks:\n");
-					foreach (Task task in blocked_tasks) {
-						task.AppendToStringBuilder (sb);
-						sb.Append ('\n');
-					}
+				foreach (Task task in future_tasks) {
+					cached_sb.Length = 0;
+					task.AppendToStringBuilder (cached_sb);
+					current_status.FutureTasks.Add (cached_sb.ToString ());
 				}
+
+				foreach (Task task in blocked_tasks) {
+					cached_sb.Length = 0;
+					task.AppendToStringBuilder (cached_sb);
+					current_status.BlockedTasks.Add (cached_sb.ToString ());
+				}
+
+				current_status.TotalTaskCount = total_executed_task_count;
+				current_status.StatusString = status_str;
+
 			}
 
+			return current_status;
+		}
+
+	}
+
+	public class SchedulerInformation {
+		[XmlAttribute]
+		public int TotalTaskCount = -1;
+
+		[XmlAttribute]
+		public string StatusString;
+
+		[XmlArray]
+		[XmlArrayItem (ElementName="PendingTask", Type=typeof (string))]
+		public ArrayList PendingTasks = new ArrayList ();
+
+		[XmlArray]
+		[XmlArrayItem (ElementName="FutureTask", Type=typeof (string))]
+		public ArrayList FutureTasks = new ArrayList ();
+
+		[XmlArray]
+		[XmlArrayItem (ElementName="BlockedTask", Type=typeof (string))]
+		public ArrayList BlockedTasks = new ArrayList ();
+
+		private static StringBuilder sb = new StringBuilder ();
+
+		public string ToHumanReadableString ()
+		{
+			sb.Length = 0;
+
+			sb.Append ("Scheduler:\n");
+			sb.Append ("Count: ").Append (TotalTaskCount);
 			sb.Append ('\n');
+
+			if (StatusString != null)
+				sb.Append ("Status: ").Append (StatusString).Append ('\n');
+
+			int pos = 1;
+			sb.Append ("\nPending Tasks:\n");
+			if (PendingTasks != null && PendingTasks.Count > 0) {
+				foreach (string task in PendingTasks) {
+					sb.Append (pos).Append (' ').Append (task).Append ('\n');
+					++pos;
+				}
+			} else
+				sb.Append ("Scheduler queue is empty.\n");
+
+
+			if (FutureTasks != null && FutureTasks.Count > 0) {
+				sb.Append ("\nFuture Tasks:\n");
+				foreach (string task in FutureTasks)
+					sb.Append (task).Append ('\n');
+			}
+
+			if (BlockedTasks != null && BlockedTasks.Count > 0) {
+				sb.Append ("\nBlocked Tasks:\n");
+				foreach (string task in BlockedTasks)
+					sb.Append (task).Append ('\n');
+			}
+
 			return sb.ToString ();
 		}
 	}
@@ -1112,17 +1171,17 @@ namespace Beagle.Util {
 				Reschedule = true;
 		}
 
-		static void BeginTaskGroup ()
+		private static void BeginTaskGroup ()
 		{
 			Console.WriteLine ("--- Begin Task Group!");
 		}
 
-		static void EndTaskGroup ()
+		private static void EndTaskGroup ()
 		{
 			Console.WriteLine ("--- End Task Group!");
 		}
 
-		static void Main ()
+		public static void Main ()
 		{
 			Scheduler sched = Scheduler.Global;
 
