@@ -71,6 +71,12 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		public abstract Indexable GetNextIndexable ();
 		public abstract void Checkpoint ();
 
+
+		protected double progress_percent;
+		public double ProgressPercent {
+			get { return progress_percent; }
+		}
+
 		public void PostFlushHook ()
 		{
 			Checkpoint ();
@@ -104,10 +110,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 					
 		protected void CrawlFinished ()
 		{
-			// FIXME: This is a little sketchy
-			this.queryable.FileAttributesStore.AttachLastWriteTime (this.CrawlFile.FullName, DateTime.UtcNow);
-
-			this.queryable.SetGeneratorProgress (this, 100);
+			this.queryable.FileAttributesStore.AttachLastWriteTime (this.CrawlFile.FullName, File.GetLastWriteTimeUtc (this.CrawlFile.FullName));
+			this.queryable.RemoveGenerator (this);
 		}
 
 		public string StatusName {
@@ -199,7 +203,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		{
 			if (this.account_name == null) {
 				if (!Setup ()) {
-					this.queryable.RemoveGeneratorProgress (this);
+					this.queryable.RemoveGenerator (this);
 					return false;
 				}
 			}
@@ -210,12 +214,17 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				try {
 					InitializeGMime ();
 				} catch (Exception e) {
-					Logger.Log.Warn ("Caught exception trying to initalize gmime:");
-					Logger.Log.Warn (e);
+					Logger.Log.Warn (e, "Caught exception trying to initalize gmime:");
 					return false;
 				}
 
 				this.mbox_fd = Mono.Unix.Native.Syscall.open (this.mbox_info.FullName, Mono.Unix.Native.OpenFlags.O_RDONLY);
+
+				if (this.mbox_fd < 0) {
+					Log.Error ("Unable to open {0}: {1}", this.mbox_info.FullName, Mono.Unix.Native.Stdlib.strerror (Mono.Unix.Native.Stdlib.GetLastError ()));
+					return false;
+				}
+
 				this.mbox_stream = new GMime.StreamFs (this.mbox_fd);
 				this.mbox_stream.Seek ((int) this.MboxLastOffset);
 				this.mbox_parser = new GMime.Parser (this.mbox_stream);
@@ -389,7 +398,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				progress = String.Format (" ({0}/{1} bytes {2:###.0}%)",
 							  offset, this.file_size, 100.0 * offset / this.file_size);
 
-				this.queryable.SetGeneratorProgress (this, (int) (100.0 * offset / this.file_size));
+				this.progress_percent = 100.0 * offset / this.file_size;
 			}
 
 			Logger.Log.Debug ("{0}: indexed {1} messages{2}",
@@ -417,7 +426,6 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		private ImapBackendType backend_type;
 		private Camel.Summary summary;
 		private IEnumerator summary_enumerator;
-		private ICollection accounts;
 		private string folder_cache_name;
 		private Hashtable mapping;
 		private ArrayList deleted_list;
@@ -466,8 +474,10 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			string imap_start = dir_name.Substring (imap_start_idx);
 			this.imap_name = imap_start.Substring (0, imap_start.IndexOf ('/'));
 
+			ICollection accounts = null;
+
 			try {
-				this.accounts = (ICollection) GConfThreadHelper.Get ("/apps/evolution/mail/accounts");
+				accounts = (ICollection) GConfThreadHelper.Get ("/apps/evolution/mail/accounts");
 			} catch (Exception ex) {
 				Logger.Log.Warn ("Caught exception in Setup(): " + ex.Message);
 				Logger.Log.Warn ("There are no configured evolution accounts, ignoring {0}", this.imap_name);
@@ -475,10 +485,10 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			}
 
 			// This should only happen if we shut down while waiting for the GConf results to come back.
-			if (this.accounts == null)
+			if (accounts == null)
 				return false;
 
-			foreach (string xml in this.accounts) {
+			foreach (string xml in accounts) {
 				XmlDocument xmlDoc = new XmlDocument ();
 
 				xmlDoc.LoadXml (xml);
@@ -612,7 +622,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		{
 			if (this.account_name == null) {
 				if (!Setup ()) {
-					this.queryable.RemoveGeneratorProgress (this);
+					this.queryable.RemoveGenerator (this);
 					return false;
 				}
 			}
@@ -627,7 +637,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				// Check to see if we even need to bother walking the summary
 				if (cache_loaded && this.queryable.FileAttributesStore.IsUpToDate (this.CrawlFile.FullName)) {
 					Logger.Log.Debug ("{0}: summary has not been updated; crawl unncessary", this.folder_name);
-					this.queryable.RemoveGeneratorProgress (this);
+					this.queryable.RemoveGenerator (this);
 					return false;
 				}
 			}
@@ -639,9 +649,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 					else
 						this.summary = Camel.Summary.LoadImap4Summary (this.summary_info.FullName);
 				} catch (Exception e) {
-					Logger.Log.Warn ("Unable to index {0}: {1}", this.folder_name,
-									 e.Message);
-					this.queryable.RemoveGeneratorProgress (this);
+					Logger.Log.Warn (e, "Unable to index {0}:", this.folder_name);
+					this.queryable.RemoveGenerator (this);
 					return false;
 				}
 			}
@@ -796,10 +805,9 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (!have_content) {
 					indexable.AddProperty (Property.NewUnsearched ("fixme:to", ia.ToString (false)));
-					if (ia.AddressType != GMime.InternetAddressType.Group) {
+					if (ia.AddressType != GMime.InternetAddressType.Group)
 						indexable.AddProperty (Property.New ("fixme:to_address", ia.Addr));
-						indexable.AddProperty (Property.New ("fixme:to_sanitized", StringFu.SanitizeEmail (ia.Addr)));
-					}
+
 					indexable.AddProperty (Property.New ("fixme:to_name", ia.Name));
 				}
 
@@ -812,10 +820,9 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (!have_content) {
 					indexable.AddProperty (Property.NewUnsearched ("fixme:cc", ia.ToString (false)));
-					if (ia.AddressType != GMime.InternetAddressType.Group) {
+					if (ia.AddressType != GMime.InternetAddressType.Group)
 						indexable.AddProperty (Property.New ("fixme:cc_address", ia.Addr));
-						indexable.AddProperty (Property.New ("fixme:cc_sanitized", StringFu.SanitizeEmail (ia.Addr)));
-					}
+
 					indexable.AddProperty (Property.New ("fixme:cc_name", ia.Name));
 				}
 
@@ -828,10 +835,9 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (!have_content) {
 					indexable.AddProperty (Property.NewUnsearched ("fixme:from", ia.ToString (false)));
-					if (ia.AddressType != GMime.InternetAddressType.Group) {
+					if (ia.AddressType != GMime.InternetAddressType.Group)
 						indexable.AddProperty (Property.New ("fixme:from_address", ia.Addr));
-						indexable.AddProperty (Property.New ("fixme:from_sanitized", StringFu.SanitizeEmail (ia.Addr)));
-					}
+
 					indexable.AddProperty (Property.New ("fixme:from_name", ia.Name));
 				}
 
@@ -889,7 +895,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 								  this.summary.header.count,
 								  100.0 * this.count / this.summary.header.count);
 
-					this.queryable.SetGeneratorProgress (this, (int) (100.0 * this.count / this.summary.header.count));
+					this.progress_percent = 100.0 * this.count / this.summary.header.count;
 
 				}
 				
