@@ -83,7 +83,7 @@ class ExtractContentTool {
 		Console.WriteLine ();
 
 		if (filter.ChildIndexables != null && filter.ChildIndexables.Count > 0) {
-			Console.WriteLine ("Child indexables:");
+			Console.WriteLine ("Child indexables ({0}):", filter.ChildIndexables.Count);
 
 			foreach (Indexable i in filter.ChildIndexables)
 				Console.WriteLine ("  {0}", i.Uri);
@@ -95,18 +95,18 @@ class ExtractContentTool {
 		ArrayList prop_array = new ArrayList (indexable.Properties);
 		prop_array.Sort ();
 
-		bool first;
-		first = true;
+		bool first = true;
+
+		Console.WriteLine ("Properties:");
+
+		if (indexable.ValidTimestamp)
+			Console.WriteLine ("  Timestamp = {0}", DateTimeUtil.ToString (indexable.Timestamp));
 
 		foreach (Beagle.Property prop in prop_array) {
-			if (first) {
-				Console.WriteLine ("Properties:");
-				first = false;
-			}
 			Console.WriteLine ("  {0} = {1}", prop.Key, prop.Value);
 		}
-		if (! first)
-			Console.WriteLine ();
+
+		Console.WriteLine ();
 
 		if (indexable.NoContent)
 			return;
@@ -155,8 +155,16 @@ class ExtractContentTool {
 		if (stream != null)
 			stream.Close ();
 
-		if (show_children && filter.ChildIndexables != null) {
+		// Clean up any temporary files associated with filtering this indexable.
+		indexable.Cleanup ();
+
+		if (filter.ChildIndexables != null) {
 			foreach (Indexable i in filter.ChildIndexables) {
+				if (! show_children) {
+					i.Cleanup ();
+					continue;
+				}
+
 				i.StoreStream ();
 				Display (i);
 			}
@@ -248,10 +256,12 @@ class ExtractContentTool {
 		if (writer != null)
 			writer.Close ();
 
+		GLib.MainLoop main_loop = new GLib.MainLoop ();
+
 		if (Environment.GetEnvironmentVariable ("BEAGLE_TEST_MEMORY") != null) {
 			GC.Collect ();
-			GLib.Timeout.Add (1000, delegate() { Gtk.Application.Quit (); return false; });
-			Gtk.Application.Run ();
+			GLib.Timeout.Add (1000, delegate() { main_loop.Quit (); return false; });
+			main_loop.Run ();
 		}
 
 		return 0;
