@@ -40,9 +40,8 @@ namespace Beagle.Daemon.LifereaQueryable {
 	[QueryableFlavor (Name="Liferea", Domain=QueryDomain.Local, RequireInotify=false)]
 	public class LifereaQueryable : LuceneFileQueryable {
 
-		private static Logger log = Logger.Get ("LifereaQueryable");
-
 		string liferea_dir;
+		internal string icon_dir;
 
 		private XmlSerializer serializer = null;
 		public XmlSerializer Serializer {
@@ -62,6 +61,7 @@ namespace Beagle.Daemon.LifereaQueryable {
 		{
 			liferea_dir = Path.Combine (PathFinder.HomeDir, ".liferea");
 			liferea_dir = Path.Combine (liferea_dir, "cache");
+			icon_dir = Path.Combine (liferea_dir, "favicons");
 			liferea_dir = Path.Combine (liferea_dir, "feeds");
 		}
 
@@ -96,9 +96,8 @@ namespace Beagle.Daemon.LifereaQueryable {
                                 fsw.EnableRaisingEvents = true;
 			}
 
-                        log.Info ("Scanning Liferea feeds...");
+                        Log.Info ("Scanning Liferea feeds...");
 
-			State = QueryableState.Crawling;
 			Stopwatch stopwatch = new Stopwatch ();
 			stopwatch.Start ();
 
@@ -108,9 +107,8 @@ namespace Beagle.Daemon.LifereaQueryable {
 				IndexSingleFeed (file.FullName);
 			}
 
-			State = QueryableState.Idle;
 			stopwatch.Stop ();
-                        log.Info ("{0} files will be parsed (scanned in {1})", count, stopwatch);
+                        Log.Info ("{0} files will be parsed (scanned in {1})", count, stopwatch);
 		}
 
 		private bool CheckForExistence ()
@@ -155,7 +153,7 @@ namespace Beagle.Daemon.LifereaQueryable {
 
 		private void IndexSingleFeed (string filename) {
 			if (ThisScheduler.ContainsByTag (filename)) {
-				Logger.Log.Debug ("Not adding task for already running task: {0}", filename);
+				Log.Debug ("Not adding task for already running task: {0}", filename);
 				return;
 			}
 
@@ -168,7 +166,7 @@ namespace Beagle.Daemon.LifereaQueryable {
 		}
 
 		private void Removefeed_file (string file) {
-			Logger.Log.Debug ("Removing Liferea feed_file:" + file);
+			Log.Debug ("Removing Liferea feed_file:" + file);
 			Uri uri = UriFu.PathToFileUri (file);
 			Scheduler.Task task = NewRemoveTask (uri);
 			task.Priority = Scheduler.Priority.Immediate;
@@ -184,6 +182,7 @@ namespace Beagle.Daemon.LifereaQueryable {
 	 */
 	public class FeedIndexableGenerator : IIndexableGenerator {
 		private string feed_file;
+		private string icon_file = null;
 		private LifereaQueryable queryable;
 		
 		private XmlTextReader reader;
@@ -200,6 +199,12 @@ namespace Beagle.Daemon.LifereaQueryable {
 			this.feed_file = feed_file;
 			this.serializer = queryable.Serializer;
 			ReadFeedHeader ();
+
+			// Set icon file
+			string file_name = Path.GetFileNameWithoutExtension (feed_file);
+			this.icon_file = this.queryable.icon_dir;
+			this.icon_file = Path.Combine (icon_file, file_name);
+			this.icon_file = Path.ChangeExtension (icon_file, "png");
 		}
 
 		public void PostFlushHook ()
@@ -224,7 +229,7 @@ namespace Beagle.Daemon.LifereaQueryable {
 				return;
 			}
 			try {
-				Logger.Log.Debug ("Opening liferea feed file: {0}", feed_file);
+				Log.Debug ("Opening liferea feed file: {0}", feed_file);
 				reader = new XmlTextReader (feed_file);
 				reader.WhitespaceHandling = WhitespaceHandling.None;
 				
@@ -257,7 +262,7 @@ namespace Beagle.Daemon.LifereaQueryable {
 					}
 				} while (!reader.EOF && reader.NodeType == XmlNodeType.Element);
 			} catch (XmlException ex) {
-				Logger.Log.Debug ("Invalid feed file: " + ex.Message);
+				Log.Warn (ex, "Caught exception parsing feed file:");
 				is_valid_file = false;
 				reader.Close ();
 			}
@@ -282,8 +287,8 @@ namespace Beagle.Daemon.LifereaQueryable {
 			}
 
 			if (current_item == null) {
-				//Logger.Log.Debug ("LifereaQ: Probably no more feeds left in " + feed_file);
-				//Logger.Log.Debug ("Causing string = " + itemString);
+				//Log.Debug ("LifereaQ: Probably no more feeds left in " + feed_file);
+				//Log.Debug ("Causing string = " + itemString);
 				current_item = null;
 				is_valid_file = false;
 				reader.Close ();
@@ -311,7 +316,7 @@ namespace Beagle.Daemon.LifereaQueryable {
 			indexable.MimeType = "text/html";
 			indexable.HitType = "FeedItem";
 
-			DateTime date = new DateTime (1970, 1, 1);
+			DateTime date = DateTimeUtil.UnixToDateTimeUtc (0);
 			date = date.AddSeconds (current_item.Timestamp);
 			indexable.Timestamp = date;				
 
@@ -331,7 +336,10 @@ namespace Beagle.Daemon.LifereaQueryable {
 			indexable.AddProperty (Property.NewKeyword ("dc:identifier", current_item.Source));
 			indexable.AddProperty (Property.NewKeyword ("dc:source", feed_source));
 			indexable.AddProperty (Property.New ("dc:publisher", publisher));
-				
+
+			if (File.Exists (icon_file))
+				indexable.AddProperty (Property.NewUnsearched ("fixme:cachedimg", icon_file));
+
 			StringReader reader = new StringReader (current_item.Description);
 			indexable.SetTextReader (reader);
 
