@@ -54,19 +54,38 @@ namespace Beagle.Daemon {
 
 		object flush_lock = new object ();
 
-		public LuceneIndexingDriver (string index_name, int minor_version) : base (index_name, minor_version)
+		public LuceneIndexingDriver (string index_name, int minor_version, bool build_usercache) 
+			: base (index_name, minor_version)
 		{
 			if (Exists ())
 				Open ();
 			else
 				Create ();
+
+			if (build_usercache)
+				text_cache = TextCache.UserCache;
 		}
 
-		public LuceneIndexingDriver (string index_name) : this (index_name, 0)
-		{ }
+		public LuceneIndexingDriver (string index_name, int minor_version)
+			: this (index_name, minor_version, true) { }
+		
+		public LuceneIndexingDriver (string index_name, bool build_usercache)
+			: this (index_name, 0, build_usercache) { }
+
+		public LuceneIndexingDriver (string index_name) 
+			: this (index_name, 0, true) { }
 	
 		////////////////////////////////////////////////////////////////
 
+		// We use this in the index helper so that we can report what's
+		// going on if the helper spins the CPU.  The method will be
+		// called with null parameters after filtering has finished.
+
+		public delegate void FileFilterDelegate (Uri display_uri, Filter filter);
+		public FileFilterDelegate FileFilterNotifier = null;
+
+		////////////////////////////////////////////////////////////////
+
 		//
 		// Implementation of the IIndexer interface
 		//
@@ -96,9 +115,12 @@ namespace Beagle.Daemon {
 			// property change requests.
 
 			LNS.BooleanQuery prop_change_query = null;
+			LNS.BooleanQuery prop_change_children_query = null;
 			int delete_count = 0;
 
-			foreach (Indexable indexable in request.Indexables) {
+			ICollection request_indexables = request.Indexables;
+
+			foreach (Indexable indexable in request_indexables) {
 
 				switch (indexable.Type) {
 
@@ -106,7 +128,7 @@ namespace Beagle.Daemon {
 				case IndexableType.Remove:
 
 					string uri_str;
-					uri_str = UriFu.UriToSerializableString (indexable.Uri);
+					uri_str = UriFu.UriToEscapedString (indexable.Uri);
 
 					Logger.Log.Debug ("-{0}", indexable.DisplayUri);
 					
@@ -135,9 +157,13 @@ namespace Beagle.Daemon {
 					break;
 
 				case IndexableType.PropertyChange:
-					if (prop_change_query == null)
+					if (prop_change_query == null) {
 						prop_change_query = new LNS.BooleanQuery ();
+						prop_change_children_query = new LNS.BooleanQuery ();
+					}
+
 					prop_change_query.Add (UriQuery ("Uri", indexable.Uri), false, false);
+					prop_change_children_query.Add (UriQuery ("ParentUri", indexable.Uri), false, false);
 					break;
 				}
 			}
@@ -152,6 +178,7 @@ namespace Beagle.Daemon {
 			// store them in a hash table for use later.  Then we
 			// delete the current secondary documents.
 			Hashtable prop_change_docs = null;
+			Hashtable prop_change_children_docs = null;
 			if (prop_change_query != null) {
 				prop_change_docs = UriFu.NewHashtable ();
 
@@ -165,15 +192,15 @@ namespace Beagle.Daemon {
 				delete_terms = new ArrayList ();
 
 				int N = hits.Length ();
+				Document doc;
 				for (int i = 0; i < N; ++i) {
-					Document doc;
 					doc = hits.Doc (i);
 					
 					string uri_str;
 					uri_str = doc.Get ("Uri");
 
 					Uri uri;
-					uri = UriFu.UriStringToUri (uri_str);
+					uri = UriFu.EscapedStringToUri (uri_str);
 					prop_change_docs [uri] = doc;
 						
 					Term term;
@@ -185,6 +212,36 @@ namespace Beagle.Daemon {
 
 				foreach (Term term in delete_terms)
 					secondary_reader.Delete (term);
+
+				// Step #2.5: Find all child indexables for this document
+				// Store them to send them later as IndexerChildIndexablesReceipts
+				prop_change_children_docs = UriFu.NewHashtable ();
+
+				hits = secondary_searcher.Search (prop_change_children_query);
+				N = hits.Length ();
+
+				for (int i = 0; i < N; ++i) {
+					doc = hits.Doc (i);
+					
+					string uri_str, parent_uri_str;
+					uri_str = doc.Get ("Uri");
+					parent_uri_str = doc.Get ("ParentUri");
+
+					Uri uri, parent_uri;
+					uri = UriFu.EscapedStringToUri (uri_str);
+					parent_uri = UriFu.EscapedStringToUri (parent_uri_str);
+
+					if (! prop_change_children_docs.Contains (parent_uri)) {
+						ArrayList c_list = new ArrayList ();
+						prop_change_children_docs [parent_uri] = c_list;
+					}
+
+					ArrayList children_list = (ArrayList) prop_change_children_docs [parent_uri];
+					children_list.Add (uri);
+				}
+
+				secondary_searcher.Close ();
+
 			}
 
 			// We are now done with the readers, so we close them.
@@ -205,7 +262,7 @@ namespace Beagle.Daemon {
 			primary_writer = new IndexWriter (PrimaryStore, IndexingAnalyzer, false);
 			secondary_writer = null;
 
-			foreach (Indexable indexable in request.Indexables) {
+			foreach (Indexable indexable in request_indexables) {
 				
 				if (indexable.Type == IndexableType.Remove)
 					continue;
@@ -230,6 +287,12 @@ namespace Beagle.Daemon {
 						secondary_writer = new IndexWriter (SecondaryStore, IndexingAnalyzer, false);
 					secondary_writer.AddDocument (new_doc);
 
+					// Add children property change indexables...
+					AddChildrenPropertyChange (
+						prop_change_children_docs,
+						indexable,
+						receipt_queue);
+
 					continue; // ...and proceed to the next Indexable
 				}
 
@@ -241,16 +304,21 @@ namespace Beagle.Daemon {
 				Logger.Log.Debug ("+{0}", indexable.DisplayUri);
 
 				Filter filter = null;
-				
+
+				if (FileFilterNotifier != null)
+					FileFilterNotifier (indexable.DisplayUri, null); // We don't know what filter yet.
+
 				// If we have content, try to find a filter
 				// which we can use to process the indexable.
 				try {
 					FilterFactory.FilterIndexable (indexable, text_cache, out filter);
 				} catch (Exception e) {
-					Logger.Log.Error ("Unable to filter {0} (mimetype={1})", indexable.DisplayUri, indexable.MimeType);
-					Logger.Log.Error (e);
+					Logger.Log.Error (e, "Unable to filter {0} (mimetype={1})", indexable.DisplayUri, indexable.MimeType);
 					indexable.NoContent = true;
 				}
+
+				if (FileFilterNotifier != null)
+					FileFilterNotifier (indexable.DisplayUri, filter); // Update with our filter
 					
 				Document primary_doc = null, secondary_doc = null;
 
@@ -264,8 +332,7 @@ namespace Beagle.Daemon {
 					// and try again -- that way it will at least end up in the index,
 					// even if we don't manage to extract the fulltext.
 
-					Logger.Log.Debug ("First attempt to index {0} failed", indexable.DisplayUri);
-					Logger.Log.Debug (ex);
+					Logger.Log.Debug (ex, "First attempt to index {0} failed", indexable.DisplayUri);
 					
 					indexable.NoContent = true;
 						
@@ -273,8 +340,7 @@ namespace Beagle.Daemon {
 						BuildDocuments (indexable, out primary_doc, out secondary_doc);
 						primary_writer.AddDocument (primary_doc);
 					} catch (Exception ex2) {
-						Logger.Log.Debug ("Second attempt to index {0} failed, giving up...", indexable.DisplayUri);
-						Logger.Log.Debug (ex2);
+						Logger.Log.Debug (ex2, "Second attempt to index {0} failed, giving up...", indexable.DisplayUri);
 					}
 				}
 				
@@ -288,11 +354,15 @@ namespace Beagle.Daemon {
 
 					// Create a receipt containing any child indexables.
 					if (filter.ChildIndexables.Count > 0) {
+						Log.Debug ("Generated {0} child indexable{1} from {2} (filtered with {3})", filter.ChildIndexables.Count, filter.ChildIndexables.Count > 1 ? "s" : "", indexable.DisplayUri, r.FilterName);
 						IndexerChildIndexablesReceipt cr;
 						cr = new IndexerChildIndexablesReceipt (indexable, filter.ChildIndexables);
 						receipt_queue.Add (cr);
 					}
 				}
+
+				if (FileFilterNotifier != null)
+					FileFilterNotifier (null, null); // reset
 				
 				if (secondary_doc != null) {
 					if (secondary_writer == null)
@@ -311,11 +381,15 @@ namespace Beagle.Daemon {
 				text_cache.CommitTransaction ();
 
 			if (request.OptimizeIndex) {
+				Stopwatch watch = new Stopwatch ();
 				Logger.Log.Debug ("Optimizing {0}", IndexName);
+				watch.Start ();
 				primary_writer.Optimize ();
 				if (secondary_writer == null)
 					secondary_writer = new IndexWriter (SecondaryStore, IndexingAnalyzer, false);
 				secondary_writer.Optimize ();
+				watch.Stop ();
+				Logger.Log.Debug ("{0} optimized in {1}", IndexName, watch);
 			}
 			
 			// Step #4. Close our writers and return the events to
@@ -332,6 +406,35 @@ namespace Beagle.Daemon {
 			
 			return receipt_array;
 		}
+
+		// Since some parent properties maybe stored in child properties
+		// as parent: property, any property change should be propagated
+		// to all its children as well.
+		private void AddChildrenPropertyChange (
+				Hashtable children_docs,
+				Indexable parent,
+				ArrayList receipt_queue)
+		{
+			if (! children_docs.Contains (parent.Uri))
+				return;
+
+			ArrayList children_list = (ArrayList) children_docs [parent.Uri];
+			IndexerChildIndexablesReceipt child_r;
+			child_r = new IndexerChildIndexablesReceipt ();
+			ArrayList child_indexable_list = new ArrayList ();
+
+			foreach (Uri uri in children_list) {
+				Indexable child_indexable;
+				child_indexable = new Indexable (IndexableType.PropertyChange, uri);
+				Log.Debug ("Creating property change child indexable for {1} (parent {0})", parent.Uri, uri);
+
+				child_indexable.SetChildOf (parent);
+				child_indexable_list.Add (child_indexable);
+			}
+
+			child_r.Children = child_indexable_list;
+			receipt_queue.Add (child_r);
+		}
 		
 		////////////////////////////////////////////////////////////////
 
