@@ -38,6 +38,7 @@ namespace Beagle.Daemon {
 	public class RemoteIndexer : IIndexer {
 
 		static string helper_path;
+		static int helper_pid = -1;
 
 		string remote_index_name;
 		int remote_index_minor_version = 0;
@@ -153,7 +154,7 @@ namespace Beagle.Daemon {
 						Logger.Log.Debug ("InnerException is IOException -- we probably need to launch a helper");
 						need_helper = true;
 					} else {
-						Logger.Log.Debug ("Exception was unexpected.  Details: {0}", ex);
+						Logger.Log.Debug (ex, "Exception was unexpected.");
 					}
 				}
 
@@ -246,7 +247,20 @@ namespace Beagle.Daemon {
 					throw new Exception (String.Format ("Couldn't launch helper process {0}", p.Id));
 
 				Logger.Log.Debug ("Found IndexHelper ({0}) in {1}", p.Id, watch);
+				helper_pid = p.Id;
 			}
 		}
+
+		static public void SignalRemoteIndexer (Mono.Unix.Native.Signum signal)
+		{
+			// No helper running right now
+			if (helper_pid == -1 || ! CheckHelper ()) {
+				helper_pid = -1;
+				return;
+			}
+
+			Log.Debug ("Forwarding signal {0} ({1}) to index helper (pid {2})", (int) signal, signal, helper_pid);
+			Mono.Unix.Native.Syscall.kill (helper_pid, signal);
+		}
 	}
 }
