@@ -1,3 +1,9 @@
+//
+// Search.cs
+//
+// Copyright (c) 2006 Novell, Inc.
+//
+
 using System;
 using System.Collections;
 
@@ -13,28 +19,35 @@ namespace Search {
 
 	public class MainWindow : Window {
 
-		Search.GroupView view;
-		Search.Entry entry;
-		Gtk.Button button;
-		Search.Spinner spinner;
-		Gtk.Tooltips tips;
-		Gtk.Notebook pages;
-		Search.Pages.QuickTips quicktips;
-		Search.Pages.RootUser rootuser;
-		Search.Pages.StartDaemon startdaemon;
-		Search.Pages.NoMatch nomatch;
-		Search.Panes panes;
-		Search.Tray.TrayIcon tray;
+		private Gtk.Button button;
+		private Gtk.Tooltips tips;
+		private Gtk.Notebook pages;
+		private Gtk.Statusbar statusbar;
 
-		uint timeout;
+		private Search.UIManager uim;
+		private Search.NotificationArea notification_area;
+		private Search.GroupView view;
+		private Search.Entry entry;
+		private Search.Spinner spinner;
+		private Search.Panes panes;
+		private Search.Tray.TrayIcon tray;
 
-		string queryText;
-		Beagle.Query currentQuery;
-		Search.ScopeType scope = ScopeType.Everywhere;
-		Search.SortType sort = SortType.Modified;
-		Search.TypeFilter filter = null;
+		private Search.Pages.QuickTips quicktips;
+		private Search.Pages.RootUser rootuser;
+		private Search.Pages.StartDaemon startdaemon;
+		private Search.Pages.NoMatch nomatch;
 
-		XKeybinder keybinder = new XKeybinder ();
+		private uint timeout;
+
+		private string query_text;
+		private Beagle.Query current_query;
+		private Search.ScopeType scope = ScopeType.Everything;
+		private Search.SortType sort = SortType.Modified;
+		private Search.TypeFilter filter = null;
+		private bool show_details = true;
+		private int total_matches = -1;
+
+		private XKeybinder keybinder = new XKeybinder ();
 
 		public static bool IconEnabled = false;
 
@@ -76,10 +89,7 @@ namespace Search {
 					break;
 
 				case "--autostarted":
-					if (! Conf.Searching.Autostart) {
-						Console.WriteLine ("beagle-search: Autostarting is disabled, not starting");
-						Environment.Exit (0);
-					}
+					// FIXME: This option is deprecated and will be removed in a future release.
 					break;
 
 				// Ignore session management
@@ -125,10 +135,9 @@ namespace Search {
 
 		public MainWindow () : base (WindowType.Toplevel)
 		{
-			Title = "Desktop Search";
+			Title = Catalog.GetString ("Desktop Search");
 			Icon = Beagle.Images.GetPixbuf ("system-search.png");
 
-			BorderWidth = 3;
 			DefaultWidth = 700;
 			DefaultHeight = 550;
 			DeleteEvent += OnWindowDelete;
@@ -136,25 +145,31 @@ namespace Search {
 			VBox vbox = new VBox ();
 			vbox.Spacing = 3;
 
-			UIManager uim = new UIManager (this);
+			uim = new UIManager (this);
 			uim.ScopeChanged += OnScopeChanged;
 			uim.SortChanged += OnSortChanged;
+			uim.ToggleDetails += OnToggleDetails;
 			uim.ShowQuickTips += OnShowQuickTips;
 			vbox.PackStart (uim.MenuBar, false, false, 0);
 
-			HBox padding_hbox = new HBox ();
-
 			HBox hbox = new HBox (false, 6);
 			
 			Label label = new Label (Catalog.GetString ("_Find:"));
 			hbox.PackStart (label, false, false, 0);
 			
 			entry = new Entry ();
+			entry.Activated += OnEntryActivated;
+			hbox.PackStart (entry, true, true, 0);
+
 			label.MnemonicWidget = entry;
 			uim.FocusSearchEntry += delegate () { entry.GrabFocus (); };
-			entry.Activated += OnEntryActivated;
-			entry.Changed += OnEntryChanged;
-			hbox.PackStart (entry, true, true, 0);
+
+			// The auto search after timeout feauture is now optional
+			// and can be disabled.
+			if (Beagle.Util.Conf.Searching.BeagleSearchAutoSearch) {
+				entry.Changed += OnEntryChanged;
+				entry.MoveCursor += OnEntryMoveCursor;
+			}
 
 			button = new Gtk.Button ();
 			Gtk.HBox button_hbox = new Gtk.HBox (false, 2);
@@ -170,17 +185,26 @@ namespace Search {
 			hbox.PackStart (buttonVBox, false, false, 0);
 
 			spinner = new Spinner ();
-
 			hbox.PackStart (spinner, false, false, 0);
 
+			HBox padding_hbox = new HBox ();
 			padding_hbox.PackStart (hbox, true, true, 9);
-
 			vbox.PackStart (padding_hbox, false, true, 6);
 
+			VBox view_box = new VBox (false, 3);
+			vbox.PackStart (view_box, true, true, 0);
+
+			HBox na_padding = new HBox ();
+			view_box.PackStart (na_padding, false, true, 0);
+
+			notification_area = new NotificationArea ();
+			na_padding.PackStart (notification_area, true, true, 3);
+
 			pages = new Gtk.Notebook ();
 			pages.ShowTabs = false;
 			pages.ShowBorder = false;
-			vbox.PackStart (pages, true, true, 0);
+			pages.BorderWidth = 3;
+			view_box.PackStart (pages, true, true, 0);
 
 			quicktips = new Pages.QuickTips ();
 			quicktips.Show ();
@@ -201,7 +225,11 @@ namespace Search {
 
 			view = new GroupView ();
 			view.TileSelected += ShowInformation;
+			view.CategoryToggled += OnCategoryToggled;
 			panes.MainContents = view;
+
+			this.statusbar = new Gtk.Statusbar ();
+			vbox.PackEnd (this.statusbar, false, false, 0);
 			
 			Add (vbox);
 
@@ -227,23 +255,48 @@ namespace Search {
 			} else {
 				ShowAll ();
 			}
-		}
 
-		Gtk.Widget oldFocus;
+			StartCheckingIndexingStatus ();
+		}
 
 		private void SetWindowTitle (string query)
 		{
-			Title = String.Format ("Desktop Search: {0}", query);
+			Title = String.Format ( Catalog.GetString ("Desktop Search: {0}"), query);
 		}
 
-		private void Search (bool grabFocus)
+		private int TotalMatches {
+			get { return this.total_matches; }
+			set {
+				if (this.total_matches != -1)
+					this.statusbar.Pop (0);
+
+				this.total_matches = value;
+				
+				if (this.total_matches > -1) {
+					string message;
+					int tile_count = view.TileCount;
+
+					if (tile_count == this.total_matches)
+						message = String.Format (Catalog.GetString ("Showing all {0} matches"), this.total_matches);
+					else
+						message = String.Format (Catalog.GetString ("Showing the top {0} of {1} total matches"), view.TileCount, this.total_matches);
+
+					this.statusbar.Push (0, message);
+				}
+			}
+		}
+
+		// Whether we should grab focus from the text entry
+		private bool grab_focus;
+
+		private void Search (bool grab_focus)
 		{
 			if (timeout != 0) {
 				GLib.Source.Remove (timeout);
 				timeout = 0;
 			}
 
-			string query = queryText = entry.Text;
+			string query = query_text = entry.Text;
 			if (query == null || query == "")
 				return;
 
@@ -261,17 +314,20 @@ namespace Search {
 			view.Sort = sort;
 			pages.CurrentPage = pages.PageNum (panes);
 
-			oldFocus = grabFocus ? Focus : null;
+			this.grab_focus = grab_focus;
 
 			try {
-				if (currentQuery != null) {
-					currentQuery.HitsAddedEvent -= OnHitsAdded;
-					currentQuery.HitsSubtractedEvent -= OnHitsSubtracted;
-					currentQuery.Close ();
+				if (current_query != null) {
+					TotalMatches = -1;
+					current_query.HitsAddedEvent -= OnHitsAdded;
+					current_query.HitsSubtractedEvent -= OnHitsSubtracted;
+					current_query.Close ();
 				}
 
-				currentQuery = new Query ();
-				currentQuery.AddDomain (QueryDomain.Neighborhood);
+				TotalMatches = 0;
+
+				current_query = new Query ();
+				current_query.AddDomain (QueryDomain.Neighborhood);
 
 				// Don't search documentation by default
 				QueryPart_Property part = new QueryPart_Property ();
@@ -279,16 +335,16 @@ namespace Search {
 				part.Type = PropertyType.Keyword;
 				part.Key = "beagle:Source";
 				part.Value = "documentation";
-				currentQuery.AddPart (part);
+				current_query.AddPart (part);
 
-				currentQuery.AddText (query);
-				currentQuery.HitsAddedEvent += OnHitsAdded;
-				currentQuery.HitsSubtractedEvent += OnHitsSubtracted;
-				currentQuery.FinishedEvent += OnFinished;
+				current_query.AddText (query);
+				current_query.HitsAddedEvent += OnHitsAdded;
+				current_query.HitsSubtractedEvent += OnHitsSubtracted;
+				current_query.FinishedEvent += OnFinished;
 
-				currentQuery.SendAsync ();
+				current_query.SendAsync ();
 				spinner.Start ();
-			} catch (Beagle.ResponseMessageException e){
+			} catch (Beagle.ResponseMessageException) {
 				pages.CurrentPage = pages.PageNum (startdaemon);
 			} catch (Exception e) {
 				Console.WriteLine ("Querying the Beagle daemon failed: {0}", e.Message);
@@ -312,6 +368,13 @@ namespace Search {
 			timeout = GLib.Timeout.Add (1000, OnEntryTimeout);
 		}
 
+		private void OnEntryMoveCursor (object obj, EventArgs args)
+		{
+			if (timeout != 0)
+				GLib.Source.Remove (timeout);
+			timeout = GLib.Timeout.Add (1000, OnEntryTimeout);
+		}
+
 		private bool OnEntryTimeout ()
 		{
 			timeout = 0;
@@ -334,24 +397,53 @@ namespace Search {
 			}
 		}
 
-		private void OnScopeChanged (Search.ScopeType newScope)
+		private void OnScopeChanged (Search.ScopeType toggled, bool active)
 		{
-			view.Scope = scope = newScope;
+			if (active)
+				view.Scope = scope = scope | toggled;
+			else
+				view.Scope = scope = scope ^ toggled;
+			
 			CheckNoMatch ();
 		}
+		
+		private void OnCategoryToggled (ScopeType toggled)
+		{
+			string name =  ScopeType.GetName (typeof (ScopeType), toggled);
+			try {
+				ToggleAction act = (ToggleAction) uim.GetAction ("/ui/MenuBar/Search/Scope/" +  name);
+				act.Active = ! act.Active;
+			}
+			catch (Exception ex) {
+				Console.WriteLine("Exception caught when trying to deactivate menu entry {0}:",name);
+				Console.WriteLine(ex);
+				return;
+			}
+		}
+		
 
 		private void OnSortChanged (Search.SortType newSort)
 		{
 			view.Sort = sort = newSort;
 		}
 
+		private void OnToggleDetails (bool active)
+		{
+			show_details = active;
+			if (panes.Details != null)
+				panes.ToggleDetails (show_details);
+			else
+				panes.ToggleDetails (false);
+		}
+
 		private void OnShowQuickTips ()
 		{
-			if (currentQuery != null) {
-				currentQuery.HitsAddedEvent -= OnHitsAdded;
-				currentQuery.HitsSubtractedEvent -= OnHitsSubtracted;
-				currentQuery.Close ();
-				currentQuery = null;
+			if (current_query != null) {
+				TotalMatches = -1;
+				current_query.HitsAddedEvent -= OnHitsAdded;
+				current_query.HitsSubtractedEvent -= OnHitsSubtracted;
+				current_query.Close ();
+				current_query = null;
 			}
 
 			pages.CurrentPage = pages.PageNum (quicktips);
@@ -359,16 +451,23 @@ namespace Search {
 
 		private void ShowInformation (Tiles.Tile tile)
 		{
-			if (tile != null)
+			if (tile != null) {
 				panes.Details = tile.Details;
-			else
+				if (tile.Details != null)
+					panes.ToggleDetails (show_details);
+				else
+					panes.ToggleDetails (false);
+			} else {
 				panes.Details = null;
+				panes.ToggleDetails (false);
+			}
 		}
 
 		private void OnFinished (FinishedResponse response)
 		{
 			spinner.Stop ();
-			view.Finished (oldFocus == Focus);
+			view.Finished (grab_focus);
+			grab_focus = false;
 
 			CheckNoMatch ();
 		}
@@ -376,7 +475,7 @@ namespace Search {
 		private void OnHitsAdded (HitsAddedResponse response)
 		{
 			foreach (Hit hit in response.Hits) {
-				Tile tile = TileActivatorOrg.MakeTile (hit, currentQuery);
+				Tile tile = TileActivatorOrg.MakeTile (hit, current_query);
 				if (tile == null)
 					continue;
 
@@ -387,6 +486,8 @@ namespace Search {
 				if (pages.CurrentPageWidget != panes)
 					pages.CurrentPage = pages.PageNum (panes);
 			}
+
+			TotalMatches += response.NumMatches;
 		}
 
 		private void OnHitsSubtracted (HitsSubtractedResponse response)
@@ -407,7 +508,7 @@ namespace Search {
 
 			if (nomatch != null)
 				nomatch.Destroy ();
-			nomatch = new Pages.NoMatch (queryText, matches == MatchType.NoneInScope);
+			nomatch = new Pages.NoMatch (query_text, matches == MatchType.NoneInScope);
 			nomatch.Show ();
 			pages.Add (nomatch);
 			pages.CurrentPage = pages.PageNum (nomatch);
@@ -418,7 +519,8 @@ namespace Search {
 		private void OnTrayActivated (object o, EventArgs args)
 		{
 			if (! Visible) {
-				ShowAll ();
+				base.ShowAll ();
+				base.Present ();
 				entry.GrabFocus ();
 			} else {
 				base.Hide ();
@@ -433,5 +535,27 @@ namespace Search {
 			entry.Text = query;
 			Search (true);
 		}
+
+		//////////////////////////////////////
+
+		private void StartCheckingIndexingStatus ()
+		{
+			InformationalMessagesRequest msg_request = new InformationalMessagesRequest ();
+			msg_request.IndexingStatusEvent += OnIndexingStatusEvent;
+			msg_request.SendAsync ();
+		}
+
+		private void OnIndexingStatusEvent (IndexingStatus status)
+		{
+			if (status == IndexingStatus.Running) {
+				NotificationMessage m = new NotificationMessage ();
+				m.Icon = Gtk.Stock.DialogInfo;
+				m.Title = Catalog.GetString ("Your data is being indexed");
+				m.Message = Catalog.GetString ("The search service is in the process of indexing your data.  Search results may be incomplete until indexing has finished.");
+				notification_area.Display (m);
+			} else {
+				notification_area.Hide ();
+			}
+		}
 	}
 }
