@@ -27,7 +27,7 @@ namespace Search.Tiles {
 
 			Title = Hit ["beagle:ExactFilename"];
 
-			if (Hit ["beagle:FilenameExtension"].Length > 0)
+			if (Hit ["beagle:FilenameExtension"] != null && Hit ["beagle:FilenameExtension"].Length > 0)
 				Description = Hit ["beagle:FilenameExtension"].Substring (1).ToUpper ();
 			
 			if (Hit ["fixme:width"] != null && Hit ["fixme:width"] != "")
@@ -45,7 +45,7 @@ namespace Search.Tiles {
 
 			// Draw the F-Spot overlay
 			if (size > 32 && Hit ["fspot:IsIndexed"] == "true") {
-				Gdk.Pixbuf icon = image.Pixbuf;
+				Gdk.Pixbuf icon = image.Pixbuf.Copy ();
 				Gdk.Pixbuf emblem = Beagle.Images.GetPixbuf ("emblem-fspot.png", 24, 24);
 
 				if (icon == null || emblem == null)
@@ -61,6 +61,7 @@ namespace Search.Tiles {
 						  emblem.Height, 0,  icon.Height - emblem.Height, 1,  1,
 						  Gdk.InterpType.Bilinear, 255);
 
+				image.Pixbuf.Dispose ();
 				image.Pixbuf = icon;
 			}
 		}
@@ -71,20 +72,16 @@ namespace Search.Tiles {
 
 			// FIXME: The icon needs a nice frame as in the spec (?)
 
-			details.AddBoldLabel (Title, 0, 1);
-			details.AddLabel (Description, 1, 1);
-			details.AddLabel ("", 2, 1);
+			details.AddTitleLabel (Title);
+			details.AddTextLabel (Description);
+			details.AddNewLine ();
 
-			details.AddLabelPair (Catalog.GetString ("Modified:"),
-					      Utils.NiceVeryLongDate (Hit.FileInfo.LastWriteTime),
-					      3, 1);
-			details.AddLabelPair (Catalog.GetString ("Full Path:"),
-					      Hit.Uri.LocalPath,
-					      4, 1);
+			details.AddLabelPair (Catalog.GetString ("Modified:"), Utils.NiceVeryLongDate (Hit.FileInfo.LastWriteTime));
+			details.AddLabelPair (Catalog.GetString ("Full Path:"), Hit.Uri.LocalPath);
 
 			if (Hit ["fspot:Description"] != null && Hit ["fspot:Description"] != "") {
-				details.AddLabel ("", 5, 1);
-				details.AddLabel (Hit ["fspot:Description"], 6, 1);
+				details.AddNewLine ();
+				details.AddTextLabel (Hit ["fspot:Description"]);
 			}
 
 			return details;
